#include "d_nxtapi.h"
#include "Font.txt"

static UBYTE pScreen[DISPLAY_HEIGHT / 8][DISPLAY_WIDTH]; 

void dSetPixel(UBYTE X,UBYTE Y)
{
  if ((X < DISPLAY_WIDTH) && (Y < DISPLAY_HEIGHT))
  {
    UBYTE *pS = (UBYTE*) pScreen;
    pS[(Y / 8) * DISPLAY_WIDTH + X] |= (1 << (Y % 8));
  }
}

void dClrPixel(UBYTE X,UBYTE Y)
{
  if ((X < DISPLAY_WIDTH) && (Y < DISPLAY_HEIGHT))
  {
    UBYTE *pS = (UBYTE*) pScreen;
    pS[(Y / 8) * DISPLAY_WIDTH + X] &= ~(1 << (Y % 8));
  }
}

const UBYTE size=4;
UBYTE sx,sy;

void dClrSuperPixel(UBYTE X,UBYTE Y)
{
    for(sx=0;sx<size;sx++)
        for(sy=0;sy<size;sy++)
            dClrPixel(X*size+sx,Y*size+sy);
}


void dSetSuperPixel(UBYTE X,UBYTE Y)
{
    for(sx=0;sx<size;sx++)
        for(sy=0;sy<size;sy++)
            dSetPixel(X*size+sx,Y*size+sy);
}

void dClearDisplay()
{
    UBYTE x = 0;
    UBYTE y = 0;
    for(x=0;x<DISPLAY_HEIGHT;x++)

    for(y=0;y< DISPLAY_HEIGHT / 8;y++)
        pScreen[y][x] = 0;
    
    for(x=0;x<5;x++)
        dDisplayUpdate(DISPLAY_HEIGHT, DISPLAY_WIDTH,(UBYTE *) pScreen);  
}

void dUpdateDisplay()
{
    dDisplayUpdate(DISPLAY_HEIGHT, DISPLAY_WIDTH,(UBYTE *) pScreen);  
}

void busyMSleep(int ms)
{
    int time;
    time = dTimerRead();
    while(dTimerRead() - time < ms);
}

static FONT *pFont = &Font;

void      dDisplayChar(UBYTE On,UBYTE X,UBYTE Y,UBYTE Char)
{
  UBYTE   *pSource;
  UBYTE   FontWidth;
  UBYTE   FontHeight;
  UBYTE   Items;
  UBYTE   Item;
  UBYTE   TmpY;


  Items          = pFont->ItemsX * pFont->ItemsY;
  Item           = Char - ' ';
  if (Item < Items)
  {
    FontWidth    = pFont->ItemPixelsX;
    pSource      = (UBYTE*)&pFont->Data[Item * FontWidth];
    while (FontWidth--)
    {
      TmpY       = 0;
      FontHeight = pFont->ItemPixelsY;
      while (FontHeight--)
      {
        if (On == TRUE)
        {
          if (((*pSource) & (1 << TmpY)))
          {
            dSetPixel(X,Y + TmpY);
          }
          else
          {
            dClrPixel(X,Y + TmpY);
          }
        }
        else
        {
          if (((*pSource) & (1 << TmpY)))
          {
            dClrPixel(X,Y + TmpY);
          }
          else
          {
            dSetPixel(X,Y + TmpY);
          }
        }
        TmpY++;
      }
      X++;
      pSource++;
    }
  }
}

void dDisplayString(UBYTE X,UBYTE Y,UBYTE *pString)
{
  UBYTE   *pSource;
  UBYTE   *pDestination;
  UBYTE   FontWidth;
  UBYTE   Line;
  UBYTE   Items;
  UBYTE   Item;


  Line         = (Y & 0xF8) / 8;
  Items        = pFont->ItemsX * pFont->ItemsY;
  pDestination = pScreen[Line * DISPLAY_WIDTH + X];

  while (*pString)
  {
    Item           = *pString - ' ';
    if (Item < Items)
    {
      FontWidth    = pFont->ItemPixelsX;
      pSource      = (UBYTE*)&pFont->Data[Item * FontWidth];
      while (FontWidth--)
      {
        *pDestination = *pSource;
        pDestination++;
        pSource++;
      }
    }
    pString++;
  }
}

void Init()
{
  HARDWAREInit;

  dDisplayInit();
  dDisplayOn(1);
  
  dIOCtrlSetPower(0);
  dIOCtrlInit();
  
  dButtonInit(2);
  
  dSoundInit();  
  dTimerInit();
  
  dClearDisplay();
}

void Shutdown()
{
    HARDWAREExit;
}

static int rnd = 0;
int Rand()
{
   if(!rnd)
       rnd = (int) dTimerRead();
   // use simple linear congruential generator
   rnd = 1664525*rnd+1013904223;
   return rnd;
}
