#include "common/AT91SAM7S256.h"
#include "stdconst.h"

#define PIT_FREQ 1           /* Hz */
#define CLOCK_FREQUENCY 480000000

typedef struct _TCB_type
{
    int cpsr;
    int ret_addr;
    int r0;
    int r1;
    int r2;
    int r3;
    int r4;
    int r5;
    int r6;
    int r7;
    int r8;
    int r9;
    int r10;
    int r11;
    int r12;
    int r13;
    int r14;
} TCB;

volatile int ms_ticks;
volatile int status;

TCB threads[3];
unsigned char stack_[4000];
int cur_Thread = 0;
int old_Thread = 0;
volatile TCB* current_thread_tcb;

extern int get_cpsr();
extern void timer_isr();

int sched_count = 0;

// update current_thread_tcb
// return pointer to tcb of next thread to run
TCB* schedule()
{
    cur_Thread++;
    if(cur_Thread>2)
        cur_Thread=0;
    current_thread_tcb = &threads[cur_Thread];
    return current_thread_tcb;//&threads[cur_Thread];
}

// initialize new thread
void create_thread(unsigned char id, void* func_entry)
{ 
    // id 0 reserved for idle thread
    if(id==0)
	return;
    threads[id].cpsr=get_cpsr();
    threads[id].ret_addr=func_entry;
    threads[id].r13=stack_+(id+1)*1000;
}

void init_tick()
{
    volatile int pimr;
    ms_ticks = 0;
    
    *AT91C_AIC_IDCR = (1 << AT91C_ID_SYS);
    AT91C_AIC_SVR[AT91C_ID_SYS] = (unsigned int) timer_isr;
    
    /* Configure Interrupt */\
    AT91C_AIC_SMR[AT91C_ID_SYS] = AT91C_AIC_SRCTYPE_INT_EDGE_TRIGGERED | AT91C_AIC_PRIOR_HIGHEST;    

    //AT91C_ID_SYS
   
    *AT91C_AIC_ICCR = (1L << AT91C_ID_SYS);       /* Clear interrupt    */
	*AT91C_PITC_PIMR = ((CLOCK_FREQUENCY / 16 / PIT_FREQ) - 1) | 0x03000000;  // set mode register
 
    *AT91C_AIC_IECR  = (1L << AT91C_ID_SYS);      /* Enable int. controller */
    
    // init first thread   
    current_thread_tcb = &threads[0];
}

unsigned int tick_count()
{
	return ms_ticks;
}
